﻿// Demo of creating an interative animation using WPF.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace BouncingBallsWPF
{

    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {        
        // We support three different speeds of animation, 10000 ticks = 1ms
        private TimeSpan SPEED_FAST   = new TimeSpan(100000);
        private TimeSpan SPEED_MEDIUM = new TimeSpan(300000);
        private TimeSpan SPEED_SLOW   = new TimeSpan(600000);
        
        private List<Ball> balls = new List<Ball>();
        private Random rand = new Random();
  
        private DispatcherTimer timer = new DispatcherTimer();

        public MainWindow()
        {
            InitializeComponent();

            timer.Tick += new EventHandler(updateTick);
            timer.Interval = SPEED_MEDIUM;
            timer.Start();
        }

        // Perdiodically fires and causes us to redraw all the balls in the canvas
        private void updateTick(object sender, EventArgs e)
        {            
            drawingCanvas.Children.Clear();

            foreach (Ball ball in balls)
            {
                ball.updatePosition(drawingCanvas.Width, drawingCanvas.Height);
                drawingCanvas.Children.Add(ball.getEllipse());
            }
        }

        // Called when they click somewhere on the Canvas.
        // Adds a new Ball object to the animation.
        private void Canvas_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            double x = e.GetPosition(this).X;
            double y = e.GetPosition(this).Y;

            System.Console.WriteLine("(" + x + "," + y + ")");

            double r = rand.Next(15) + 5;
            
            // Only create if Ball would be inside the bouncing boundaries of the canvas
            if ((x > r) && (x < drawingCanvas.Width - r) && (y > r) && (y < drawingCanvas.Height - r))
            {
                Ball ball = new Ball(x, y, r);
                balls.Add(ball);

                labelCount.Content = "" + balls.Count;
            }
        }

        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            timer.Stop();
        }

        private void ButtonSlow_Click(object sender, RoutedEventArgs e)
        {
            timer.Interval = SPEED_SLOW;
        }

        private void ButtonMedium_Click(object sender, RoutedEventArgs e)
        {
            timer.Interval = SPEED_MEDIUM;
        }

        private void ButtonFast_Click(object sender, RoutedEventArgs e)
        {
            timer.Interval = SPEED_FAST;
        }
    }
}
