﻿// Demo of bouncing balls in a Windows Forms C# app.

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Threading;

namespace BouncingBallsWindowsForms
{
    public partial class Form1 : Form
    {
        private List<Ball> balls = new List<Ball>();
        private Random rand = new Random();

        private const int SPEED_FAST = 10;
        private const int SPEED_MEDIUM = 30;
        private const int SPEED_SLOW = 60;

        private volatile int currentSpeed = SPEED_MEDIUM;
        private Thread thread = null;
        private volatile bool shutdown = false;

        public Form1()
        {
            InitializeComponent();

            thread = new Thread(new ThreadStart(worker));
            thread.Start();
        }

        private void worker()
        {
            while (!shutdown)
            {
                foreach (Ball ball in balls)
                {
                    ball.updatePosition(drawingPanel.Width, drawingPanel.Height);               
                }
                drawingPanel.Invalidate();
                Thread.Sleep(currentSpeed);
            }
        }

        private void ButtonFast_Click(object sender, EventArgs e)
        {
            currentSpeed = SPEED_FAST;
        }

        private void ButtonMedium_Click(object sender, EventArgs e)
        {
            currentSpeed = SPEED_MEDIUM;
        }

        private void ButtonSlow_Click(object sender, EventArgs e)
        {
            currentSpeed = SPEED_SLOW;
        }

        private void Panel_Paint(object sender, PaintEventArgs e)
        {
            foreach (Ball ball in balls)
            {
                ball.draw(e.Graphics);
            }
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            shutdown = true;
            thread.Join();
        }

        private void Panel_MouseUp(object sender, MouseEventArgs e)
        {
            double x = e.X;
            double y = e.Y;

            System.Console.WriteLine("(" + x + "," + y + ")");

            double r = rand.Next(15) + 5;

            // Only create if Ball would be inside the bouncing boundaries of the canvas
            if ((x > r) && (x < drawingPanel.Width - r) && (y > r) && (y < drawingPanel.Height - r))
            {
                Ball ball = new Ball(x, y, r);
                balls.Add(ball);

                labelCount.Text = "" + balls.Count;
            }
        }
    }
}
